@@ -358,7 +358,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
    * @tool: The new current tool
    *
    * The ::tool-changed signal is emitted on pen/eraser
-   * #GdkDevice<!-- -->s whenever tools enter or leave proximity.
+   * #GdkDevices whenever tools enter or leave proximity.
    *
    * Since: 3.22
    */
@@ -367,7 +367,7 @@ gdk_device_class_init (GdkDeviceClass *klass)
                   G_TYPE_FROM_CLASS (object_class),
                   G_SIGNAL_RUN_LAST,
                   0, NULL, NULL,
-                  g_cclosure_marshal_VOID__BOXED,
+                  g_cclosure_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1, GDK_TYPE_DEVICE_TOOL);
 }
 
