@@ -33,8 +33,6 @@
 
 #include "config.h"
 
-#include "gdkprivate-win32.h"
-#include "gdkdisplay-win32.h"
 #include "gdkmonitor-win32.h"
 
 #include <glib.h>
@@ -559,7 +557,6 @@ enum_monitor (HMONITOR hmonitor,
       DEVMODEW dm;
       DWORD i_monitor;
       DWORD frequency;
-      GdkWin32MonitorRotation orientation;
 
       memset (&dd, 0, sizeof (dd));
       dd.cb = sizeof (dd);
@@ -578,36 +575,12 @@ enum_monitor (HMONITOR hmonitor,
         continue;
 
       dm.dmSize = sizeof (dm);
-      dm.dmDriverExtra = 0;
-      frequency = 0;
-      orientation = GDK_WIN32_MONITOR_ROTATION_UNKNOWN;
 
       /* Grab refresh rate for this adapter while we're at it */
-      if (EnumDisplaySettingsExW (dd.DeviceName, ENUM_CURRENT_SETTINGS, &dm, 0))
-        {
-          if ((dm.dmFields & DM_DISPLAYORIENTATION) == DM_DISPLAYORIENTATION)
-            {
-              switch (dm.dmDisplayOrientation)
-                {
-                default:
-                case DMDO_DEFAULT:
-                  orientation = GDK_WIN32_MONITOR_ROTATION_0;
-                  break;
-                case DMDO_90:
-                  orientation = GDK_WIN32_MONITOR_ROTATION_90;
-                  break;
-                case DMDO_180:
-                  orientation = GDK_WIN32_MONITOR_ROTATION_180;
-                  break;
-                case DMDO_270:
-                  orientation = GDK_WIN32_MONITOR_ROTATION_270;
-                  break;
-                }
-            }
-
-          if ((dm.dmFields & DM_DISPLAYFREQUENCY) == DM_DISPLAYFREQUENCY)
-            frequency = dm.dmDisplayFrequency;
-        }
+      if (EnumDisplaySettingsW (dd.DeviceName, ENUM_CURRENT_SETTINGS, &dm))
+        frequency = dm.dmDisplayFrequency;
+      else
+        frequency = 0;
 
       /* Enumerate monitors connected to this display adapter */
       for (i_monitor = 0; TRUE; i_monitor++)
@@ -618,7 +591,6 @@ enum_monitor (HMONITOR hmonitor,
           GdkWin32Monitor *w32mon;
           GdkMonitor *mon;
           GdkRectangle rect;
-          guint scale;
 
           memset (&dd_monitor, 0, sizeof (dd_monitor));
           dd_monitor.cb = sizeof (dd_monitor);
@@ -708,50 +680,19 @@ enum_monitor (HMONITOR hmonitor,
           /* This is the reason this function exists. This data is not available
            * via other functions.
            */
-          rect.x = monitor_info.rcWork.left;
-          rect.y = monitor_info.rcWork.top;
-          rect.width = (monitor_info.rcWork.right - monitor_info.rcWork.left);
-          rect.height = (monitor_info.rcWork.bottom - monitor_info.rcWork.top);
-          /* This is temporary, scale will be applied below */
-          w32mon->work_rect = rect;
-
-          if (data->display->has_fixed_scale)
-            scale = data->display->window_scale;
-          else
-            {
-              /* First acquire the scale using the current screen */
-              scale = _gdk_win32_display_get_monitor_scale_factor (data->display, NULL, NULL, NULL);
-
-              /* acquire the scale using the monitor which the window is nearest on Windows 8.1+ */
-              if (data->display->have_at_least_win81)
-                {
-                  HMONITOR hmonitor;
-                  POINT pt;
-
-                  /* Not subtracting _gdk_offset_x and _gdk_offset_y because they will only
-                   * be added later on, in _gdk_win32_display_get_monitor_list().
-                   */
-                  pt.x = w32mon->work_rect.x + w32mon->work_rect.width / 2;
-                  pt.y = w32mon->work_rect.y + w32mon->work_rect.height / 2;
-                  hmonitor = MonitorFromPoint (pt, MONITOR_DEFAULTTONEAREST);
-                  scale = _gdk_win32_display_get_monitor_scale_factor (data->display, hmonitor, NULL, NULL);
-                }
-            }
-
-          gdk_monitor_set_scale_factor (mon, scale);
-          /* Now apply the scale to the work rectangle */
-          w32mon->work_rect.x /= scale;
-          w32mon->work_rect.y /= scale;
-          w32mon->work_rect.width /= scale;
-          w32mon->work_rect.height /= scale;
-
-          rect.x = monitor_info.rcMonitor.left / scale;
-          rect.y = monitor_info.rcMonitor.top / scale;
-          rect.width = (monitor_info.rcMonitor.right - monitor_info.rcMonitor.left) / scale;
-          rect.height = (monitor_info.rcMonitor.bottom - monitor_info.rcMonitor.top) / scale;
+          rect.x = monitor_info.rcMonitor.left;
+          rect.y = monitor_info.rcMonitor.top;
+          rect.width = monitor_info.rcMonitor.right - monitor_info.rcMonitor.left;
+          rect.height = monitor_info.rcMonitor.bottom - monitor_info.rcMonitor.top;
           gdk_monitor_set_position (mon, rect.x, rect.y);
           gdk_monitor_set_size (mon, rect.width, rect.height);
 
+          rect.x = monitor_info.rcWork.left;
+          rect.y = monitor_info.rcWork.top;
+          rect.width = monitor_info.rcWork.right - monitor_info.rcWork.left;
+          rect.height = monitor_info.rcWork.bottom - monitor_info.rcWork.top;
+          w32mon->work_rect = rect;
+
           if (monitor_info.dwFlags & MONITORINFOF_PRIMARY && i != 0)
             {
               /* Put primary monitor at index 0, just in case somebody needs
@@ -769,7 +710,6 @@ enum_monitor (HMONITOR hmonitor,
            * keep remove == TRUE and be removed further up the stack.
            */
           w32mon->remove = FALSE;
-          w32mon->orientation = orientation;
 
           /* One virtual monitor per display adapter */
           if (w32mon->madeup)
@@ -780,74 +720,6 @@ enum_monitor (HMONITOR hmonitor,
   return TRUE;
 }
 
-static void
-prune_monitors (EnumMonitorData *data)
-{
-  gint i;
-
-  for (i = 0; i < data->monitors->len; i++)
-    {
-      GdkWin32Monitor *m;
-
-      m = g_ptr_array_index (data->monitors, i);
-
-      if (m->remove)
-        g_ptr_array_remove_index (data->monitors, i--);
-    }
-}
-
-const gchar *
-_gdk_win32_monitor_get_pixel_structure (GdkMonitor *monitor)
-{
-  GdkWin32Monitor *w32_m;
-  BOOL enabled = TRUE;
-  unsigned int smoothing_orientation = FE_FONTSMOOTHINGORIENTATIONRGB;
-  UINT cleartype = FE_FONTSMOOTHINGCLEARTYPE;
-
-  g_return_val_if_fail (monitor != NULL, NULL);
-
-  w32_m = GDK_WIN32_MONITOR (monitor);
-
-  SystemParametersInfoW (SPI_GETFONTSMOOTHING, 0, &enabled, 0);
-  SystemParametersInfoW (SPI_GETFONTSMOOTHINGTYPE, 0, &cleartype, 0);
-
-  if (!enabled ||
-      (cleartype == FE_FONTSMOOTHINGSTANDARD) ||
-      !SystemParametersInfoW (SPI_GETFONTSMOOTHINGORIENTATION, 0, &smoothing_orientation, 0))
-    return "none";
-
-  if (smoothing_orientation == FE_FONTSMOOTHINGORIENTATIONBGR)
-    switch (w32_m->orientation)
-      {
-      default:
-      case GDK_WIN32_MONITOR_ROTATION_UNKNOWN:
-        return "none";
-      case GDK_WIN32_MONITOR_ROTATION_0:
-        return "bgr";
-      case GDK_WIN32_MONITOR_ROTATION_90:
-        return "vbgr";
-      case GDK_WIN32_MONITOR_ROTATION_180:
-        return "rgb";
-      case GDK_WIN32_MONITOR_ROTATION_270:
-        return "vrgb";
-      }
-  else
-    switch (w32_m->orientation)
-      {
-      default:
-      case GDK_WIN32_MONITOR_ROTATION_UNKNOWN:
-        return "none";
-      case GDK_WIN32_MONITOR_ROTATION_0:
-        return "rgb";
-      case GDK_WIN32_MONITOR_ROTATION_90:
-        return "vrgb";
-      case GDK_WIN32_MONITOR_ROTATION_180:
-        return "bgr";
-      case GDK_WIN32_MONITOR_ROTATION_270:
-        return "vbgr";
-      }
-}
-
 GPtrArray *
 _gdk_win32_display_get_monitor_list (GdkWin32Display *win32_display)
 {
@@ -869,18 +741,6 @@ _gdk_win32_display_get_monitor_list (GdkWin32Display *win32_display)
 
   EnumDisplayMonitors (NULL, NULL, enum_monitor, (LPARAM) &data);
 
-  prune_monitors (&data);
-
-  if (data.monitors->len == 0 && data.have_monitor_devices)
-    {
-      /* We thought we had monitors, but enumeration eventually failed, and
-       * we have none. Try again, this time making stuff up as we go.
-       */
-      data.have_monitor_devices = FALSE;
-      EnumDisplayMonitors (NULL, NULL, enum_monitor, (LPARAM) &data);
-      prune_monitors (&data);
-    }
-
   _gdk_offset_x = G_MININT;
   _gdk_offset_y = G_MININT;
 
@@ -891,6 +751,12 @@ _gdk_win32_display_get_monitor_list (GdkWin32Display *win32_display)
 
       m = g_ptr_array_index (data.monitors, i);
 
+      if (m->remove)
+        {
+          g_ptr_array_remove_index (data.monitors, i);
+          continue;
+        }
+
       /* Calculate offset */
       gdk_monitor_get_geometry (GDK_MONITOR (m), &rect);
       _gdk_offset_x = MAX (_gdk_offset_x, -rect.x);
