@@ -91,6 +91,7 @@ struct _GtkPlacesViewPrivate
   guint                          unmounting_mount : 1;
   guint                          fetching_networks : 1;
   guint                          loading : 1;
+  guint                          destroyed : 1;
 };
 
 static void        mount_volume                                  (GtkPlacesView *view,
@@ -387,20 +388,31 @@ activate_row (GtkPlacesView      *view,
 
 static void update_places (GtkPlacesView *view);
 
+static void
+gtk_places_view_destroy (GtkWidget *widget)
+{
+  GtkPlacesView *self = GTK_PLACES_VIEW (widget);
+  GtkPlacesViewPrivate *priv = gtk_places_view_get_instance_private (self);
+
+  priv->destroyed = 1;
+
+  g_signal_handlers_disconnect_by_func (priv->volume_monitor, update_places, widget);
+
+  g_cancellable_cancel (priv->cancellable);
+  g_cancellable_cancel (priv->networks_fetching_cancellable);
+
+  GTK_WIDGET_CLASS (gtk_places_view_parent_class)->destroy (widget);
+}
+
 static void
 gtk_places_view_finalize (GObject *object)
 {
   GtkPlacesView *self = (GtkPlacesView *)object;
   GtkPlacesViewPrivate *priv = gtk_places_view_get_instance_private (self);
 
-  g_signal_handlers_disconnect_by_func (priv->volume_monitor, update_places, object);
-
   if (priv->entry_pulse_timeout_id > 0)
     g_source_remove (priv->entry_pulse_timeout_id);
 
-  g_cancellable_cancel (priv->cancellable);
-  g_cancellable_cancel (priv->networks_fetching_cancellable);
-
   g_clear_pointer (&priv->search_query, g_free);
   g_clear_object (&priv->server_list_file);
   g_clear_object (&priv->server_list_monitor);
@@ -958,8 +970,11 @@ network_enumeration_next_files_finished (GObject      *source_object,
       g_list_free_full (detected_networks, g_object_unref);
     }
 
-  /* avoid to update widgets if the operation was cancelled in finalize */
-  if (priv->listbox != NULL)
+  g_object_unref (view);
+
+  /* avoid to update widgets if we are already destroyed
+     (and got cancelled s a result of that) */
+  if (!priv->destroyed)
     {
       update_network_state (view);
       update_loading (view);
@@ -985,6 +1000,7 @@ network_enumeration_finished (GObject      *source_object,
         g_warning ("Failed to fetch network locations: %s", error->message);
 
       g_clear_error (&error);
+      g_object_unref (GTK_PLACES_VIEW (user_data));
     }
   else
     {
@@ -1024,6 +1040,7 @@ fetch_networks (GtkPlacesView *view)
   gtk_places_view_set_fetching_networks (view, TRUE);
   update_network_state (view);
 
+  g_object_ref (view);
   g_file_enumerate_children_async (network_file,
                                    "standard::type,standard::target-uri,standard::name,standard::display-name,standard::icon",
                                    G_FILE_QUERY_INFO_NONE,
@@ -1151,6 +1168,8 @@ server_mount_ready_cb (GObject      *source_file,
   should_show = TRUE;
   error = NULL;
 
+  view = GTK_PLACES_VIEW (user_data);
+
   g_file_mount_enclosing_volume_finish (location, res, &error);
   if (error)
     {
@@ -1168,22 +1187,22 @@ server_mount_ready_cb (GObject      *source_file,
                (error->code != G_IO_ERROR_CANCELLED &&
                 error->code != G_IO_ERROR_FAILED_HANDLED))
         {
-          view = GTK_PLACES_VIEW (user_data);
           /* if it wasn't cancelled show a dialog */
           emit_show_error_message (view, _("Unable to access location"), error->message);
-          should_show = FALSE;
-        }
-      else
-        {
-          /* it was cancelled, so probably it was called during finalize, bail out. */
-          g_clear_error (&error);
-          return;
         }
+
+      /* The operation got cancelled by the user and or the error
+         has been handled already. */
       g_clear_error (&error);
     }
 
-  view = GTK_PLACES_VIEW (user_data);
   priv = gtk_places_view_get_instance_private (view);
+
+  if (priv->destroyed) {
+    g_object_unref (view);
+    return;
+  }
+
   priv->should_pulse_entry = FALSE;
 
   /* Restore from Cancel to Connect */
@@ -1207,6 +1226,7 @@ server_mount_ready_cb (GObject      *source_file,
     }
 
   update_places (view);
+  g_object_unref (view);
 }
 
 static void
@@ -1246,18 +1266,21 @@ volume_mount_ready_cb (GObject      *source_volume,
           emit_show_error_message (GTK_PLACES_VIEW (user_data), _("Unable to access location"), error->message);
           should_show = FALSE;
         }
-      else
-        {
-          /* it was cancelled, so probably it was called during finalize, bail out. */
-          g_clear_error (&error);
-          return;
-        }
 
+      /* The operation got cancelled by the user and or the error
+         has been handled already. */
       g_clear_error (&error);
     }
 
   view = GTK_PLACES_VIEW (user_data);
   priv = gtk_places_view_get_instance_private (view);
+
+  if (priv->destroyed)
+    {
+      g_object_unref(view);
+      return;
+    }
+
   priv->mounting_volume = FALSE;
   update_loading (view);
 
@@ -1277,6 +1300,7 @@ volume_mount_ready_cb (GObject      *source_volume,
     }
 
   update_places (view);
+  g_object_unref (view);
 }
 
 static void
@@ -1293,10 +1317,6 @@ unmount_ready_cb (GObject      *source_mount,
   mount = G_MOUNT (source_mount);
   error = NULL;
 
-  priv = gtk_places_view_get_instance_private (view);
-  priv->unmounting_mount = FALSE;
-  update_loading (view);
-
   g_mount_unmount_with_operation_finish (mount, res, &error);
 
   if (error)
@@ -1311,6 +1331,18 @@ unmount_ready_cb (GObject      *source_mount,
 
       g_clear_error (&error);
     }
+
+  priv = gtk_places_view_get_instance_private (view);
+
+  if (priv->destroyed) {
+    g_object_unref (view);
+    return;
+  }
+
+  priv->unmounting_mount = FALSE;
+  update_loading (view);
+
+  g_object_unref (view);
 }
 
 static gboolean
@@ -1320,7 +1352,13 @@ pulse_entry_cb (gpointer user_data)
 
   priv = gtk_places_view_get_instance_private (GTK_PLACES_VIEW (user_data));
 
-  if (priv->should_pulse_entry)
+  if (priv->destroyed)
+    {
+      priv->entry_pulse_timeout_id = 0;
+
+      return G_SOURCE_REMOVE;
+    }
+  else if (priv->should_pulse_entry)
     {
       gtk_entry_progress_pulse (GTK_ENTRY (priv->address_entry));
 
@@ -1330,7 +1368,6 @@ pulse_entry_cb (gpointer user_data)
     {
       gtk_entry_set_progress_pulse_step (GTK_ENTRY (priv->address_entry), 0.0);
       gtk_entry_set_progress_fraction (GTK_ENTRY (priv->address_entry), 0.0);
-      priv->entry_pulse_timeout_id = 0;
 
       return G_SOURCE_REMOVE;
     }
@@ -1355,6 +1392,8 @@ unmount_mount (GtkPlacesView *view,
   priv->unmounting_mount = TRUE;
   update_loading (view);
 
+  g_object_ref (view);
+
   operation = gtk_mount_operation_new (GTK_WINDOW (toplevel));
   g_mount_unmount_with_operation (mount,
                                   0,
@@ -1398,6 +1437,9 @@ mount_server (GtkPlacesView *view,
 
   g_mount_operation_set_password_save (operation, G_PASSWORD_SAVE_FOR_SESSION);
 
+  /* make sure we keep the view around for as long as we are running */
+  g_object_ref (view);
+
   g_file_mount_enclosing_volume (location,
                                  0,
                                  operation,
@@ -1430,6 +1472,9 @@ mount_volume (GtkPlacesView *view,
 
   g_mount_operation_set_password_save (operation, G_PASSWORD_SAVE_FOR_SESSION);
 
+  /* make sure we keep the view around for as long as we are running */
+  g_object_ref (view);
+
   g_volume_mount (volume,
                   0,
                   operation,
@@ -2121,6 +2166,7 @@ gtk_places_view_class_init (GtkPlacesViewClass *klass)
   object_class->get_property = gtk_places_view_get_property;
   object_class->set_property = gtk_places_view_set_property;
 
+  widget_class->destroy = gtk_places_view_destroy;
   widget_class->map = gtk_places_view_map;
 
   /**
