@@ -176,6 +176,7 @@ struct _GtkPopoverPrivate
   guint transitions_enabled : 1;
   guint state               : 2;
   guint visible             : 1;
+  guint first_frame_skipped : 1;
   gint transition_diff;
   guint tick_id;
 
@@ -551,8 +552,12 @@ show_animate_cb (GtkWidget     *widget,
   GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
   gdouble t;
 
-  gtk_progress_tracker_advance_frame (&priv->tracker,
-                                      gdk_frame_clock_get_frame_time (frame_clock));
+  if (priv->first_frame_skipped)
+    gtk_progress_tracker_advance_frame (&priv->tracker,
+                                        gdk_frame_clock_get_frame_time (frame_clock));
+  else
+    priv->first_frame_skipped = TRUE;
+
   t = gtk_progress_tracker_get_ease_out_cubic (&priv->tracker, FALSE);
 
   if (priv->state == STATE_SHOWING)
@@ -609,6 +614,7 @@ gtk_popover_start_transition (GtkPopover *popover)
   if (priv->tick_id != 0)
     return;
 
+  priv->first_frame_skipped = FALSE;
   gtk_progress_tracker_start (&priv->tracker, TRANSITION_DURATION, 0, 1.0);
   priv->tick_id = gtk_widget_add_tick_callback (GTK_WIDGET (popover),
                                                 show_animate_cb,
@@ -846,44 +852,51 @@ gtk_popover_get_gap_coords (GtkPopover      *popover,
 }
 
 static void
-gtk_popover_get_rect_coords (GtkPopover *popover,
-                             gint       *x1_out,
-                             gint       *y1_out,
-                             gint       *x2_out,
-                             gint       *y2_out)
+gtk_popover_get_rect_for_size (GtkPopover   *popover,
+                               int           popover_width,
+                               int           popover_height,
+                               GdkRectangle *rect)
 {
   GtkWidget *widget = GTK_WIDGET (popover);
-  GtkAllocation allocation;
-  gint x1, x2, y1, y2;
+  int x, y, w, h;
   GtkBorder margin;
 
-  gtk_widget_get_allocation (widget, &allocation);
   get_margin (widget, &margin);
 
-  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR)
-    x1 = gtk_widget_get_margin_start (widget);
-  else
-    x1 = gtk_widget_get_margin_end (widget);
+  x = 0;
+  y = 0;
+  w = popover_width;
+  h = popover_height;
 
-  y1 = gtk_widget_get_margin_top (widget);
-  x2 = allocation.width -
-    gtk_widget_get_margin_end (widget) + x1;
-  y2 = allocation.height -
-    gtk_widget_get_margin_bottom (widget) + y1;
+  x += MAX (TAIL_HEIGHT, margin.left);
+  y += MAX (TAIL_HEIGHT, margin.top);
+  w -= x + MAX (TAIL_HEIGHT, margin.right);
+  h -= y + MAX (TAIL_HEIGHT, margin.bottom);
 
-  x1 += MAX (TAIL_HEIGHT, margin.left);
-  y1 += MAX (TAIL_HEIGHT, margin.top);
-  x2 -= MAX (TAIL_HEIGHT, margin.right);
-  y2 -= MAX (TAIL_HEIGHT, margin.bottom);
+  rect->x = x;
+  rect->y = y;
+  rect->width = w;
+  rect->height = h;
+}
 
-  if (x1_out)
-    *x1_out = x1;
-  if (y1_out)
-    *y1_out = y1;
-  if (x2_out)
-    *x2_out = x2;
-  if (y2_out)
-    *y2_out = y2;
+static void
+gtk_popover_get_rect_coords (GtkPopover *popover,
+                             int        *x_out,
+                             int        *y_out,
+                             int        *w_out,
+                             int        *h_out)
+{
+  GtkWidget *widget = GTK_WIDGET (popover);
+  GdkRectangle rect;
+  GtkAllocation allocation;
+
+  gtk_widget_get_allocation (widget, &allocation);
+  gtk_popover_get_rect_for_size (popover, allocation.width, allocation.height, &rect);
+
+  *x_out = rect.x;
+  *y_out = rect.y;
+  *w_out = rect.width;
+  *h_out = rect.height;
 }
 
 static void
@@ -916,7 +929,7 @@ gtk_popover_fill_border_path (GtkPopover *popover,
   GtkWidget *widget = GTK_WIDGET (popover);
   GtkAllocation allocation;
   GtkStyleContext *context;
-  gint x1, y1, x2, y2;
+  int x, y, w, h;
   GtkRoundedBox box;
 
   context = gtk_widget_get_style_context (widget);
@@ -928,9 +941,9 @@ gtk_popover_fill_border_path (GtkPopover *popover,
   cairo_close_path (cr);
   cairo_fill (cr);
 
-  gtk_popover_get_rect_coords (popover, &x1, &y1, &x2, &y2);
+  gtk_popover_get_rect_coords (popover, &x, &y, &w, &h);
 
-  _gtk_rounded_box_init_rect (&box, x1, y1, x2 - x1, y2 - y1);
+  _gtk_rounded_box_init_rect (&box, x, y, w, h);
   _gtk_rounded_box_apply_border_radius_for_style (&box,
                                                   gtk_style_context_lookup_style (context),
                                                   0);
@@ -1119,7 +1132,7 @@ gtk_popover_draw (GtkWidget *widget,
   GtkWidget *child;
   GtkBorder border;
   GdkRGBA border_color;
-  gint rect_x1, rect_x2, rect_y1, rect_y2;
+  int rect_x, rect_y, rect_w, rect_h;
   gint initial_x, initial_y, final_x, final_y;
   gint gap_start, gap_end;
   GtkPositionType gap_side;
@@ -1132,14 +1145,13 @@ gtk_popover_draw (GtkWidget *widget,
 
   gtk_style_context_get_border (context, state, &border);
   gtk_popover_get_rect_coords (popover,
-                               &rect_x1, &rect_y1,
-                               &rect_x2, &rect_y2);
+                               &rect_x, &rect_y,
+                               &rect_w, &rect_h);
 
   /* Render the rect background */
   gtk_render_background (context, cr,
-                         rect_x1, rect_y1,
-                         rect_x2 - rect_x1,
-                         rect_y2 - rect_y1);
+                         rect_x, rect_y,
+                         rect_w, rect_h);
 
   if (popover->priv->widget)
     {
@@ -1151,27 +1163,27 @@ gtk_popover_draw (GtkWidget *widget,
 
       if (POS_IS_VERTICAL (gap_side))
         {
-          gap_start = initial_x - rect_x1;
-          gap_end = final_x - rect_x1;
+          gap_start = initial_x - rect_x;
+          gap_end = final_x - rect_x;
         }
       else
         {
-          gap_start = initial_y - rect_y1;
-          gap_end = final_y - rect_y1;
+          gap_start = initial_y - rect_y;
+          gap_end = final_y - rect_y;
         }
 
       /* Now render the frame, without the gap for the arrow tip */
       gtk_render_frame_gap (context, cr,
-                            rect_x1, rect_y1,
-                            rect_x2 - rect_x1, rect_y2 - rect_y1,
+                            rect_x, rect_y,
+                            rect_w, rect_h,
                             gap_side,
                             gap_start, gap_end);
     }
   else
     {
       gtk_render_frame (context, cr,
-                        rect_x1, rect_y1,
-                        rect_x2 - rect_x1, rect_y2 - rect_y1);
+                        rect_x, rect_y,
+                        rect_w, rect_h);
     }
 
   /* Clip to the arrow shape */
@@ -1207,7 +1219,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
   if (child)
     gtk_container_propagate_draw (GTK_CONTAINER (widget), child, cr);
 
-  return TRUE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static void
@@ -1303,8 +1315,8 @@ gtk_popover_get_preferred_width_for_height (GtkWidget *widget,
                                             gint      *natural_width)
 {
   GtkPopover *popover = GTK_POPOVER (widget);
-  GtkPopoverPrivate *priv = popover->priv;
   GtkWidget *child;
+  GdkRectangle child_rect;
   gint min, nat, extra, minimal_size;
   gint child_height;
   GtkBorder border, margin;
@@ -1312,10 +1324,9 @@ gtk_popover_get_preferred_width_for_height (GtkWidget *widget,
   child = gtk_bin_get_child (GTK_BIN (widget));
   min = nat = 0;
 
-  child_height = height;
+  gtk_popover_get_rect_for_size (popover, 0, height, &child_rect);
+  child_height = child_rect.height;
 
-  if (POS_IS_VERTICAL (priv->preferred_position))
-    child_height -= TAIL_HEIGHT;
 
   get_padding_and_border (widget, &border);
   get_margin (widget, &margin);
@@ -1374,8 +1385,8 @@ gtk_popover_get_preferred_height_for_width (GtkWidget *widget,
                                             gint      *natural_height)
 {
   GtkPopover *popover = GTK_POPOVER (widget);
-  GtkPopoverPrivate *priv = popover->priv;
   GtkWidget *child;
+  GdkRectangle child_rect;
   gint min, nat, extra, minimal_size;
   gint child_width;
   GtkBorder border, margin;
@@ -1383,13 +1394,12 @@ gtk_popover_get_preferred_height_for_width (GtkWidget *widget,
   child = gtk_bin_get_child (GTK_BIN (widget));
   min = nat = 0;
 
-  child_width = width;
-
-  if (!POS_IS_VERTICAL (priv->preferred_position))
-    child_width -= TAIL_HEIGHT;
-
   get_padding_and_border (widget, &border);
   get_margin (widget, &margin);
+
+  gtk_popover_get_rect_for_size (popover, width, 0, &child_rect);
+  child_width = child_rect.width;
+
   child_width -= border.left + border.right;
   minimal_size = get_minimal_size (popover, GTK_ORIENTATION_VERTICAL);
   if (child)
@@ -1460,16 +1470,16 @@ gtk_popover_size_allocate (GtkWidget     *widget,
   if (child)
     {
       GtkAllocation child_alloc;
-      gint x1, y1, x2, y2;
+      int x, y, w, h;
       GtkBorder border;
 
-      gtk_popover_get_rect_coords (popover, &x1, &y1, &x2, &y2);
+      gtk_popover_get_rect_coords (popover, &x, &y, &w, &h);
       get_padding_and_border (widget, &border);
 
-      child_alloc.x = x1 + border.left;
-      child_alloc.y = y1 + border.top;
-      child_alloc.width = (x2 - x1) - border.left - border.right;
-      child_alloc.height = (y2 - y1) - border.top - border.bottom;
+      child_alloc.x = x + border.left;
+      child_alloc.y = y + border.top;
+      child_alloc.width = w - border.left - border.right;
+      child_alloc.height = h - border.top - border.bottom;
       gtk_widget_size_allocate (child, &child_alloc);
     }
 
